/*
 * Name:    d_compumotor_trans.h 
 *
 * Purpose: Header file for MX motor driver to use multiple MX motors 
 *          controlled by the same Compumotor controller to perform a
 *          linear translation.  This driver is quick scan capable.
 *
 * Author:  William Lavender
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __D_COMPUMOTOR_TRANS_H__
#define __D_COMPUMOTOR_TRANS_H__

#include "mx_motor.h"

/* ===== MX Compumotor translation motor data structures ===== */

typedef struct {
	long flags;

	long num_motors;
	MX_RECORD **motor_record_array;

	double *motor_position_array;

	int *index_to_axis_number;
	int axis_number_to_index[MX_MAX_COMPUMOTOR_AXES];

	MX_RECORD *compumotor_interface_record;
	int controller_number;
	int num_axes;
} MX_COMPUMOTOR_TRANSLATION_MOTOR;

/* Define all of the interface functions. */

MX_API mx_status_type mxd_compumotor_trans_initialize_type( long type );
MX_API mx_status_type mxd_compumotor_trans_create_record_structures(
					MX_RECORD *record );
MX_API mx_status_type mxd_compumotor_trans_finish_record_initialization(
					MX_RECORD *record );
MX_API mx_status_type mxd_compumotor_trans_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_compumotor_trans_print_motor_structure(
					FILE *file, MX_RECORD *record );
MX_API mx_status_type mxd_compumotor_trans_read_parms_from_hardware(
					MX_RECORD *record );
MX_API mx_status_type mxd_compumotor_trans_write_parms_to_hardware(
					MX_RECORD *record );
MX_API mx_status_type mxd_compumotor_trans_open( MX_RECORD *record );
MX_API mx_status_type mxd_compumotor_trans_close( MX_RECORD *record );

MX_API mx_status_type mxd_compumotor_trans_motor_is_busy( MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_move_absolute( MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_get_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_set_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_soft_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_immediate_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_positive_limit_hit(MX_MOTOR *motor);
MX_API mx_status_type mxd_compumotor_trans_negative_limit_hit(MX_MOTOR *motor);
MX_API mx_status_type mxd_compumotor_trans_find_home_position(MX_MOTOR *motor);
MX_API mx_status_type mxd_compumotor_trans_constant_velocity_move(
							MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_get_parameter( MX_MOTOR *motor );
MX_API mx_status_type mxd_compumotor_trans_set_parameter( MX_MOTOR *motor );

extern MX_RECORD_FUNCTION_LIST mxd_compumotor_trans_record_function_list;
extern MX_MOTOR_FUNCTION_LIST mxd_compumotor_trans_motor_function_list;

extern long mxd_compumotor_trans_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_compumotor_trans_rfield_def_ptr;

#define MXD_COMPUMOTOR_TRANS_MOTOR_STANDARD_FIELDS \
  {-1, -1, "flags", MXFT_HEX, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_COMPUMOTOR_TRANSLATION_MOTOR, flags),\
	{0}, NULL, MXFF_IN_DESCRIPTION}, \
  \
  {-1, -1, "num_motors", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, \
		offsetof(MX_COMPUMOTOR_TRANSLATION_MOTOR, num_motors), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }, \
  \
  {-1, -1, "motor_record_array", MXFT_RECORD, NULL, 1, {MXU_VARARGS_LENGTH}, \
	MXF_REC_TYPE_STRUCT, \
		offsetof(MX_COMPUMOTOR_TRANSLATION_MOTOR, motor_record_array),\
	{sizeof(MX_RECORD *)}, NULL, \
		(MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY | MXFF_VARARGS)}

#endif /* __D_COMPUMOTOR_TRANS_H__ */
