/* 
 * Name:    d_network_mca.h
 *
 * Purpose: Header file for MX network multichannel analyzers.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2000-2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __D_NETWORK_MCA_H__
#define __D_NETWORK_MCA_H__

#include "mx_mca.h"

#define MXD_NMCA_TICKS_PER_SECOND	(50.0)

typedef struct {
	MX_RECORD *server_record;
	char remote_record_name[ MXU_RECORD_NAME_LENGTH+1 ];
} MX_NETWORK_MCA;

#define MXD_NETWORK_MCA_STANDARD_FIELDS \
  {-1, -1, "server_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof( MX_NETWORK_MCA, server_record ), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "remote_record_name", MXFT_STRING, \
	NULL, 1, {MXU_RECORD_NAME_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_NETWORK_MCA, remote_record_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }

MX_API mx_status_type mxd_network_mca_initialize_type( long type );
MX_API mx_status_type mxd_network_mca_create_record_structures(
							MX_RECORD *record );
MX_API mx_status_type mxd_network_mca_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxd_network_mca_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_network_mca_print_structure( FILE *file,
							MX_RECORD *record );
MX_API mx_status_type mxd_network_mca_read_parms_from_hardware(
							MX_RECORD *record );
MX_API mx_status_type mxd_network_mca_write_parms_to_hardware(
							MX_RECORD *record );
MX_API mx_status_type mxd_network_mca_open( MX_RECORD *record );
MX_API mx_status_type mxd_network_mca_close( MX_RECORD *record );

MX_API mx_status_type mxd_network_mca_start( MX_MCA *mca );
MX_API mx_status_type mxd_network_mca_stop( MX_MCA *mca );
MX_API mx_status_type mxd_network_mca_read( MX_MCA *mca );
MX_API mx_status_type mxd_network_mca_clear( MX_MCA *mca );
MX_API mx_status_type mxd_network_mca_busy( MX_MCA *mca );
MX_API mx_status_type mxd_network_mca_get_parameter( MX_MCA *mca );
MX_API mx_status_type mxd_network_mca_set_parameter( MX_MCA *mca );

extern MX_RECORD_FUNCTION_LIST mxd_network_mca_record_function_list;
extern MX_MCA_FUNCTION_LIST mxd_network_mca_mca_function_list;

extern long mxd_network_mca_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_network_mca_rfield_def_ptr;

#endif /* __D_NETWORK_MCA_H__ */
