/*
 * Name:    d_network_motor.h 
 *
 * Purpose: Header file for MX motor driver for motors controlled
 *          via an MX network server.
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 1999, 2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __D_NETWORK_MOTOR_H__
#define __D_NETWORK_MOTOR_H__

#include "mx_motor.h"
#include "mx_net.h"

typedef struct {
	MX_RECORD *server_record;
	char remote_record_name[ MXU_RECORD_NAME_LENGTH+1 ];
} MX_NETWORK_MOTOR;

/* Define all of the interface functions. */

MX_API mx_status_type mxd_network_motor_initialize_type( long type );
MX_API mx_status_type mxd_network_motor_create_record_structures(
					MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_finish_record_initialization(
					MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_print_motor_structure(
					FILE *file, MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_read_parms_from_hardware(
					MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_write_parms_to_hardware(
					MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_open( MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_close( MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_resynchronize( MX_RECORD *record );
MX_API mx_status_type mxd_network_motor_motor_is_busy( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_move_absolute( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_get_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_set_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_soft_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_immediate_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_positive_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_negative_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_find_home_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_constant_velocity_move(
							MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_get_parameter( MX_MOTOR *motor );
MX_API mx_status_type mxd_network_motor_set_parameter( MX_MOTOR *motor );

MX_API mx_status_type mxd_network_motor_get_pointers( MX_MOTOR *motor,
				MX_NETWORK_MOTOR **network_motor,
				const char *calling_fname );

extern MX_RECORD_FUNCTION_LIST mxd_network_motor_record_function_list;
extern MX_MOTOR_FUNCTION_LIST mxd_network_motor_motor_function_list;

extern long mxd_network_motor_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_network_motor_rfield_def_ptr;

#define MXD_NETWORK_MOTOR_STANDARD_FIELDS \
  {-1, -1, "server_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_NETWORK_MOTOR, server_record), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }, \
  \
  {-1, -1, "remote_record_name", MXFT_STRING, \
	NULL, 1, {MXU_RECORD_NAME_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_NETWORK_MOTOR, remote_record_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }

#endif /* __D_NETWORK_MOTOR_H__ */
