/* 
 * Name:    d_xia_dxp.h
 *
 * Purpose: Header file for the X-Ray Instrumentation Associates Mesa2x
 *          Data Server used by the DXP-2X multichannel analyzer.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __D_XIA_DXP_H__
#define __D_XIA_DXP_H__

#include "mx_mca.h"

#define MX_XIA_DXP_CLOCK_TICK		(400.0e-9)	/* in seconds */

/* The following flags are used by the "PRESET" MCA parameter. */

#define MXF_XIA_PRESET_NONE		0
#define MXF_XIA_PRESET_REAL_TIME	1
#define MXF_XIA_PRESET_LIVE_TIME	2
#define MXF_XIA_PRESET_OUTPUT_EVENTS	3
#define MXF_XIA_PRESET_INPUT_COUNTS	4

typedef struct {
	MX_RECORD *record;

	MX_RECORD *xia_dxp_record;
	int system_channel;

	int dxp_module;

	int crate;
	int slot;
	int dxp_channel;

	double input_count_rate;
	double output_count_rate;

	mx_uint32_type num_fast_peaks;
	mx_uint32_type num_events;
	mx_uint32_type num_underflows;
	mx_uint32_type num_overflows;

	mx_uint32_type old_preset_type;
	mx_uint32_type old_preset_high_order;
	mx_uint32_type old_preset_low_order;

	mx_status_type (*is_busy)( MX_MCA *mca,
					int *busy_flag,
					int debug_flag);

	mx_status_type (*read_parameter)( MX_MCA *mca,
					char *parameter_name,
					mx_uint32_type *value_ptr,
					int debug_flag );

	mx_status_type (*write_parameter)( MX_MCA *mca,
					char *parameter_name,
					mx_uint32_type value,
					int debug_flag );

	mx_status_type (*write_parameter_to_all_channels)( MX_MCA *mca,
					char *parameter_name,
					mx_uint32_type value,
					int debug_flag );

	mx_status_type (*start_run)( MX_MCA *mca,
					int clear_flag,
					int debug_flag );

	mx_status_type (*stop_run)( MX_MCA *mca,
					int debug_flag );

	mx_status_type (*read_spectrum)( MX_MCA *mca,
					int debug_flag );

	mx_status_type (*read_statistics)( MX_MCA *mca,
					int debug_flag );

#if HAVE_XIA_XERXES

	unsigned short num_parameters;
	unsigned short *parameter_array;

	unsigned int num_baseline_bins;
	unsigned short *baseline_array;
	
	int use_mca_channel_array;

	unsigned int num_spectrum_bins;
	unsigned long *spectrum_array;

#endif /* HAVE_XIA_XERXES */

} MX_XIA_DXP_MCA;

#define MXD_XIA_DXP_STANDARD_FIELDS \
  {-1, -1, "xia_dxp_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof( MX_XIA_DXP_MCA, xia_dxp_record ), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "system_channel", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof( MX_XIA_DXP_MCA, system_channel ), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}

MX_API mx_status_type mxd_xia_dxp_initialize_type( long type );
MX_API mx_status_type mxd_xia_dxp_create_record_structures( MX_RECORD *record );
MX_API mx_status_type mxd_xia_dxp_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxd_xia_dxp_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_xia_dxp_print_structure( FILE *file,
							MX_RECORD *record );
MX_API mx_status_type mxd_xia_dxp_read_parms_from_hardware( MX_RECORD *record );
MX_API mx_status_type mxd_xia_dxp_write_parms_to_hardware( MX_RECORD *record );
MX_API mx_status_type mxd_xia_dxp_open( MX_RECORD *record );
MX_API mx_status_type mxd_xia_dxp_close( MX_RECORD *record );

MX_API mx_status_type mxd_xia_dxp_start( MX_MCA *mca );
MX_API mx_status_type mxd_xia_dxp_stop( MX_MCA *mca );
MX_API mx_status_type mxd_xia_dxp_read( MX_MCA *mca );
MX_API mx_status_type mxd_xia_dxp_clear( MX_MCA *mca );
MX_API mx_status_type mxd_xia_dxp_busy( MX_MCA *mca );
MX_API mx_status_type mxd_xia_dxp_get_parameter( MX_MCA *mca );
MX_API mx_status_type mxd_xia_dxp_set_parameter( MX_MCA *mca );

extern MX_RECORD_FUNCTION_LIST mxd_xia_dxp_record_function_list;
extern MX_MCA_FUNCTION_LIST mxd_xia_dxp_mca_function_list;

extern long mxd_xia_dxp_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_xia_dxp_rfield_def_ptr;

#endif /* __D_XIA_DXP_H__ */
