/*
 * Name:   i_xia_mds.h
 *
 * Purpose: Header for MX driver for a generic interface to the X-Ray 
 *          Instrumentation Associates Mesa2x Data Server used by 
 *          the DXP-2X multichannel analyzer.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __I_XIA_MDS_H__
#define __I_XIA_MDS_H__

/* The message buffer length is expressed in 32 bit integers. */

#define MX_XIA_MDS_MAXIMUM_BUFFER_LENGTH	33000

#define MX_XIA_MDS_HEADER_LENGTH		12
#define MX_XIA_MDS_MAXIMUM_BODY_LENGTH \
		( MX_XIA_MDS_MAXIMUM_BUFFER_LENGTH - MX_XIA_MDS_HEADER_LENGTH )

/* Define the data structures used by the XIA MDS driver. */

typedef struct {
	MX_RECORD *record;

	char hostname[ MXU_HOSTNAME_LENGTH + 1 ];
	int port_number;
	char config_filename[ MXU_FILENAME_LENGTH + 1 ];

	MX_SOCKET socket;

	mx_uint32_type request_id;

	mx_uint32_type *send_buffer;
	mx_uint32_type *receive_buffer;

	unsigned long num_mcas;
	MX_RECORD **mca_record_array;

	unsigned long *restore_configuration_response;
} MX_XIA_MDS;

/* These defines are used to identify word offsets in network packets. */

#define MXH_XIA_MESSAGE_SIZE		0
#define MXH_XIA_REQUEST_ID		1
#define MXH_XIA_REQUEST_TYPE		2
#define MXH_XIA_ERROR_CODE		2
#define MXH_XIA_MESSAGE_BODY		3

/* The following flags are used by mxi_xia_mds_command(). */

#define MXF_XIA_INVALID_COMMAND		0
#define MXF_XIA_ECHO_TEST		1
#define MXF_XIA_RESTORE_CONFIGURATION	2
#define MXF_XIA_READ_STATUS		3
#define MXF_XIA_READ_PARAMETER		4
#define MXF_XIA_WRITE_PARAMETER		5
#define MXF_XIA_START_RUN		6
#define MXF_XIA_STOP_RUN		7
#define MXF_XIA_READ_DATA		8
#define MXF_XIA_CLEAR_EXTERNAL_MEMORY	9
#define MXF_XIA_READ_WRITE_SCA_SETTINGS	10

/* This define is used by the commands that can accept an all channels value. */

#define MXF_XIA_ALL_CHANNELS		(0xFFFFFFFF)   /* equivalent to -1 */

/* The following flags are used by mxi_xia_mds_read_data(). */

#define MXF_XIA_READ_MCA_SPECTRUM	10
#define MXF_XIA_READ_STATISTICS		11
#define MXF_XIA_READ_BASELINE_SPECTRUM	20
#define MXF_XIA_READ_ADC_TRACE		30
#define MXF_XIA_READ_EXTERNAL_MEMORY	40

#define MXI_XIA_MDS_STANDARD_FIELDS \
  {-1, -1, "hostname", MXFT_STRING, NULL, 1, {MXU_HOSTNAME_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_XIA_MDS, hostname), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "port_number", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_XIA_MDS, port_number), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "config_filename", MXFT_STRING, NULL, 1, {MXU_FILENAME_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_XIA_MDS, config_filename), \
	{0}, NULL, MXFF_IN_DESCRIPTION}

MX_API mx_status_type mxi_xia_mds_initialize_type( long type );
MX_API mx_status_type mxi_xia_mds_create_record_structures( MX_RECORD *record );
MX_API mx_status_type mxi_xia_mds_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxi_xia_mds_delete_record( MX_RECORD *record );
MX_API mx_status_type mxi_xia_mds_read_parms_from_hardware(MX_RECORD *record);
MX_API mx_status_type mxi_xia_mds_write_parms_to_hardware(MX_RECORD *record);
MX_API mx_status_type mxi_xia_mds_open( MX_RECORD *record );
MX_API mx_status_type mxi_xia_mds_close( MX_RECORD *record );

extern MX_RECORD_FUNCTION_LIST mxi_xia_mds_record_function_list;

extern long mxi_xia_mds_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxi_xia_mds_rfield_def_ptr;

/* === Driver specific functions === */

MX_API mx_status_type mxi_xia_mds_echo_test( MX_RECORD *record,
					char *test_data,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_restore_configuration( MX_RECORD *record,
					char *configuration_filename,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_is_busy( MX_MCA *mca,
					int *busy_flag,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_read_parameter( MX_MCA *mca,
					char *parameter_name,
					mx_uint32_type *value_ptr,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_write_parameter( MX_MCA *mca,
					char *parameter_name,
					mx_uint32_type value,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_write_parameter_to_all_channels(
					MX_MCA *mca,
					char *parameter_name,
					mx_uint32_type value,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_start_run( MX_MCA *mca,
					int clear_flag,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_stop_run( MX_MCA *mca,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_read_spectrum( MX_MCA *mca,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_read_statistics( MX_MCA *mca,
					int debug_flag );

/*------*/

MX_API mx_status_type mxi_xia_mds_command( MX_XIA_MDS *xia_mds,
					int debug_flag );

MX_API mx_status_type mxi_xia_mds_pack_string( 
					mx_uint32_type *destination_buffer,
					unsigned long *num_words_packed,
					unsigned long maximum_num_words,
					char *source_string );

MX_API mx_status_type mxi_xia_mds_display_packed_string(
					mx_uint32_type *packed_string_buffer,
					unsigned long buffer_length_in_words );

#endif /* __I_XIA_MDS_H__ */
